/*
 * Copyright (C) 2010 Neil Jagdish Patel
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <neil.patel@canonical.com>
 */

namespace Gwibber
{
    public class Entry : Gtk.VBox
    {
        private Service service;

        private InputTextView text_view;

        public Entry ()
        {
            Object ();
        }

        construct
        {
            this.service = new Service ();

            this.text_view = new InputTextView (this.service);
            this.add (this.text_view);
            this.text_view.show ();
        }
    }

    public class InputTextView : Gtk.TextView
    {
        public Service service { get; construct; }

        private Gdk.Color base_color;
        private Gdk.Color error_color;

        bool last_was_shortened = false;

        public InputTextView (Service service)
        {
            Object (service:service,
                    border_width:10,
                    accepts_tab:true,
                    editable:true,
                    cursor_visible:true,
                    wrap_mode:Gtk.WrapMode.WORD_CHAR,
                    left_margin:2,
                    right_margin:2,
                    pixels_above_lines:2,
                    pixels_below_lines:2);
        }

        construct
        {
            this.base_color = this.get_style ().base[Gtk.StateType.NORMAL];
            Gdk.Color.parse ("indianred", out this.error_color);

            this.get_buffer ().changed.connect (this.on_text_changed);
            this.get_buffer ().insert_text.connect (this.on_text_inserted);

            //this.set_sensitive (this.service.is_connected ());
        }

        private void on_text_changed ()
        {
            var chars = this.get_buffer ().get_char_count ();

            this.modify_base (Gtk.StateType.NORMAL,
                        chars > 140 ? this.error_color : this.base_color);
        }

        private void on_text_inserted (Gtk.TextIter iter, string text, int len)
        {
            if (this.last_was_shortened == false
                && len > 30
                && text != null
                && text[0:4] == "http")
            {
                var buf = this.get_buffer ();
                Signal.stop_emission_by_name (buf, "insert-text") ;
                var shrt = this.service.shorten (text);
                this.last_was_shortened = true;
                buf.insert (iter, shrt, -1);
            }
            else
            {
                this.last_was_shortened = false;
            }
        }
    }
}

