/*
 * Copyright (C) 2010 Neil Jagdish Patel
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * version 3.0 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3.0 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * Authored by Neil Jagdish Patel <neil.patel@canonical.com>
 * Authored by Ken VaDine <ken.vandine@canonical.com>
 */

namespace Gwibber
{
    public class Service : Object
    {
        private const string conn_name  = "com.Gwibber.Connection";
        private const string conn_path  = "/com/gwibber/Connection";
        private const string conn_iface = "com.Gwibber.Connection";

        private const string service_name  = "com.Gwibber.Service";
        private const string service_path  = "/com/gwibber/Service";
        private const string service_iface = "com.Gwibber.Service";

        private const string url_name  = "com.Gwibber.URLShorten";
        private const string url_path  = "/com/gwibber/URLShorten";
        private const string url_iface = "com.Gwibber.URLShorten";

        private DBus.Connection?     connection;
        private dynamic DBus.Object? conn_service;
        private dynamic DBus.Object? service;
        private dynamic DBus.Object? url_service;

        public signal void connection_changed (bool is_online);

        public Service ()
        {
        }

        construct
        {
            try
            {
                this.connection = DBus.Bus.get (DBus.BusType.SESSION);

                this.service = this.connection.get_object (this.service_name,
                                                           this.service_path,
                                                           this.service_iface);

                this.conn_service = this.connection.get_object (this.conn_name,
                                                                this.conn_path,
                                                                this.conn_iface);

                this.conn_service.ConnectionOffline += (this.connection_offline);
                this.conn_service.ConnectionOnline  += (this.connection_online);

                this.url_service = this.connection.get_object (this.url_name,
                                                               this.url_path,
                                                               this.url_iface);
            }
            catch (Error e)
            {
                warning ("Unable to get Gwibber service");
            }
        }

        /**
         * com.Gwibber.Service
         **/
        public void refresh () throws DBus.Error
        {
            try {
                this.service.Refresh ();
            } catch (DBus.Error e) {
                error (e.message);
            }
        }

        public void SendMessage (string message) throws DBus.Error
        {
            try {
                service.SendMessage (message);
            } catch (DBus.Error e) {
                error (e.message);
            }
        }

        public List GetServices () throws DBus.Error
        {
            List<string> ret = new List<string> ();
            try {
                string services = service.GetServices ();
                stdout.printf (services);
                var parser = new Json.Parser ();
                parser.load_from_data (services);
                var root_object = parser.get_root ().get_object ();
                foreach (var serviceNode in root_object.get_members()) {
                    stdout.printf ("Service: %s\n", serviceNode);
                }
                ret = root_object.get_members();
            
            } catch (DBus.Error e) {
                error (e.message);
            }
        return ret;
        }

        public List GetFeatures () throws DBus.Error
        {
            List<string> ret = new List<string> ();
            try {
                string features = service.GetFeatures ();
                stdout.printf (features);
                var parser = new Json.Parser ();
                parser.load_from_data (features);
                var root_object = parser.get_root ().get_object ();
                foreach (var featureNode in root_object.get_members()) {
                    stdout.printf ("Feature: %s\n", featureNode);
                }
                ret = root_object.get_members();

            } catch (DBus.Error e) {
                error (e.message);
            }
        return ret;
        }
    
        public string GetAccounts () throws DBus.Error
        {
            try {
                return service.GetAccounts ();
            } catch (DBus.Error e) {
                error (e.message);
            }
        
            return "";
        }

        /**
         * com.Gwibber.URLShorten
         **/
        public string shorten (string url)
        {
            try
            {
                return this.url_service.Shorten (url);
            }
            catch (Error e)
            {
                return "Failed";
            }
        }   

        /**
         * com.Gwibber.Connection
         **/
        public bool is_connected ()
        {
            return this.conn_service.isConnected ();
        }

        private void connection_offline ()
        {
            this.connection_changed (false);
        }

        private void connection_online ()
        {
            this.connection_changed (true);
        }
    }
}
